import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TM {
    private String codierung;
    private String[][] funktionen;
    private int aktuellerZustand;
    private int endZustand;
    private List<Character> band;
    private int leseKopf;
    private Map<Character, Integer> alphabet;
    private int numberOfSteps;


    public TM(String codierung, String bandInitialisierung) {
        numberOfSteps = 0;
        leseKopf = 15;
        band = new ArrayList<>();
        this.codierung = codierung;
        endZustand = 2;
        alphabet = new HashMap<>();
        initBand(bandInitialisierung);
        initAlphabet();
        parse();

    }

    public void start(boolean onlyResult) {
        while (!checkWin() ) {
            try {
                calcOneStep();
                appendToBand();
                if(!onlyResult){
                    System.out.println("Step: "+numberOfSteps);
                    System.out.println("Aktueller zustand: Q" + aktuellerZustand);
                    printBand();
                }
            } catch (TmException e) {
                e.printStackTrace();
                break;
            }
        }
        System.out.println("Finished:");
        System.out.println("Zustand: Q" + aktuellerZustand);
        System.out.println("Step: "+numberOfSteps);
        printBand();
    }

    private void calcOneStep() throws TmException {
        boolean found = false;
        for (String[] str : funktionen) {
            if (str[0].length() == aktuellerZustand && str[1].length() == alphabet.get(band.get(leseKopf))) {
                found = true;
                writeOnBand(str);
                numberOfSteps++;
                break;
            }
        }
        if (!found) {
            throw new TmException("Couldn't find matching function for Character on band");
        }
    }

    private void writeOnBand(String[] str) {
        for (Character c : alphabet.keySet()) {
            if (alphabet.get(c) == str[3].length()) {
                band.set(leseKopf, c);
            }
        }
        if (str[4].length() == 2) {
            leseKopf += 1;
        } else {
            leseKopf -= 1;
        }
        aktuellerZustand = str[2].length();
    }

    public void printBand() {
        for(int i = 0; i < band.size(); i++) {
            if(i == leseKopf){
                System.out.print("\u001B[31m" + band.get(i) + "\u001B[0m");
            } else {
                System.out.print(band.get(i));
            }
        }
        System.out.println();
    }

    private boolean checkWin() {
        return aktuellerZustand == endZustand;
    }

    private void appendToBand() {
        if(leseKopf - 15 < 0) {
            for (int i = 0; i < 15 - leseKopf; i++) {
                band.add(i,'_');
            }
        }
        if ((band.size() - leseKopf) < 15 ) {
            for (int i = 0; i < 15 - (band.size() - leseKopf); i++) {
                band.add(band.size(),'_');
            }
        }
    }

    private void parse() {
        String[] liste = codierung.substring(1).split("11");
        funktionen = new String[liste.length][4];
        for (int i = 0; i < liste.length; i++) {
            funktionen[i] = liste[i].split("1");
        }
        aktuellerZustand = funktionen[0][0].length();
    }

    private void initBand(String bandInit) {
        for (int i = 0; i < 15; i++) {
            band.add('_');
        }
        for (int i = 0; i < bandInit.length(); i++) {
            band.add(bandInit.charAt(i));
        }
        for (int i = 0; i < 15; i++) {
            band.add('_');
        }
    }

    private void initAlphabet() {
        alphabet.put('0', 1);
        alphabet.put('1', 2);
        alphabet.put('_', 3);
        alphabet.put('X', 4);
    }

    private class TmException extends Exception {

        private TmException(String msg) {
            super(msg);
        }
    }

}
